'// Directory Size Reporting Utility
'// Copyright 2004 by Casey Sprangel, IFX Group.
'// Written in PowerBasic Console Compiler <http://www.powerbasic.com>

'// This program returns a total byte count of the file within the specified directory.

'//-----------------------------------------------------
#INCLUDE "win32api.inc"                                 '// get it, we will need it

%show_none = 0                                          '// display flag values for use later
%show_raw = 1
%show_human = 2

'//-----------------------------------------------------
FUNCTION GetFolderSize (path AS STRING) AS QUAD         '// this could potentially be a big number
  '// This function will return the total number of bytes stored
  '// in a subdirectory and any child subdirecories.
  DIM buf AS LOCAL STRING                               '// temp string buffer
  DIM lpath AS LOCAL STRING                             '// holds the local directory path we are scanning
  DIM ftotal AS LOCAL QUAD                              '// total bytes we find
  DIM fhandle AS LOCAL LONG                             '// temp file handle for getting the info
  DIM f AS LOCAL WIN32_FIND_DATA                        '// this structure will hold the file info
  DIM sd AS LOCAL STRING                                '// holds any subdirectories we find
  lpath = RTRIM$(path, "\") + "\"                       '// make sure our path has a slash
  buf = DIR$(lpath + "*.*", 23)                         '// find first file
  IF LEN(buf) > 0 THEN                                  '// if not empty
    buf = lpath + buf                                   '// add our full path the the front of the filename
  END IF
  DO WHILE LEN(buf) > 0                                 '// loop through all of the files we find
    IF (GETATTR(buf) AND %SUBDIR) <> 0 THEN             '// this is a subdirectory?
      sd = LTRIM$(sd + CHR$(0) + buf, CHR$(0))          '// append the new subdirectory to our null delim list
    ELSE                                                '// this is a file
      fHandle = FindFirstFile(BYVAL STRPTR(buf), f)
      IF fHandle <> %INVALID_HANDLE_VALUE THEN          '// we found the file
        FindClose fHandle
        ftotal = ftotal + VAL("&H" + HEX$(f.nFileSizeHigh, 8) + HEX$(f.nFileSizeLow, 8)) '// get file size
      END IF
    END IF
    buf = DIR$                                          '// find next file
    IF LEN(buf) > 0 THEN                                '// if not empty
      buf = lpath + buf                                 '// add our path back to the front of the file name we found
    END IF
  LOOP
  DO WHILE LEN(sd) > 0                                  '// did we find any subdirectories?
    buf = LEFT$(sd, INSTR(sd + CHR$(0),CHR$(0)) - 1)    '// get the first path
    sd = MID$(sd, INSTR(sd + CHR$(0), CHR$(0)) + 1)     '// clip off the path we got
    IF RIGHT$(buf,2) <> "\." AND RIGHT$(buf,3) <> "\.." THEN '// not place holder paths?
      ftotal = ftotal + GetFolderSize(buf)              '// get file sizes from this directory too
    END IF
  LOOP
  FUNCTION = ftotal
END FUNCTION

'//-----------------------------------------------------
FUNCTION exist(file AS STRING) AS LONG                  '// find out if a file exists without using FindFirst/FindNext calls
  DIM dummy AS LOCAL LONG                               '// a simple place holder variable
  dummy = GETATTR(file)                                 '// get the attribute flags for the requested file - generates an error if the file does not exist
  FUNCTION = (ERRCLEAR = 0)                             '// return true if the ERR is Zero, otherwise return false - in both cases clear the error out of the stack
END FUNCTION

'//-----------------------------------------------------
FUNCTION PBMAIN()                                       '// program logic starts here
  DIM buf AS STRING                                     '// holds the path
  DIM path AS STRING                                    '// our working path
  DIM i AS QUAD                                         '// temp pointer and holds our return value
  DIM t AS LONG                                         '// flag to show the time in the output
  DIM d AS LONG                                         '// flag to show the date in the output
  DIM show AS LONG                                      '// flag for output display style
  show = %show_none                                     '// default output display style
  PRINT "IFX Group DirSize 20040627"                    '// show our title line - can not be redirected
  buf = COMMAND$                                        '// get the command line
  IF buf = "" OR INSTR(buf, ANY "?*") > 0 THEN          '// is it empty or does it have wildcards?
    STDOUT "This tool calculates and reports the total size of all files stored in a"
    STDOUT "directory and all subdirectories below it."
    STDOUT ""
    STDOUT "Syntax:  DIRSIZE <path> [/R] [/H] [/D] [/T]"
    STDOUT ""
    STDOUT "The path is any valid UNC path available to this workstation. The total number"
    STDOUT "of bytes will be returned to standard output that can be redirected to a file"
    STDOUT "or to other programs if desired. Wildcards are not supported."
    STDOUT ""
    STDOUT "  /R will show path and raw number of bytes found."
    STDOUT "  /H will show path and human readable bytes found."
    STDOUT "  /T will show the current time."
    STDOUT "  /D will show the current date."
    EXIT FUNCTION                                       '// leave program
  END IF
  i = INSTR(UCASE$(buf), "/T")                          '// look for /T (time) command line switch
  IF i > 0 THEN
    t = -1                                              '// show time
    buf = LEFT$(buf, i-1) + MID$(buf, i + 2)            '// cut out the command line switch
  END IF
  i = INSTR(UCASE$(buf), "/D")                          '// look for /D (date) command line switch
  IF i > 0 THEN
    d = -1                                              '// show date
    buf = LEFT$(buf, i-1) + MID$(buf, i + 2)            '// cut out the command line switch
  END IF
  i = INSTR(UCASE$(buf), "/R")                          '// look for /R (raw number output) command line switch
  IF i > 0 THEN
    show = %show_raw                                    '// raw output
    buf = LEFT$(buf, i-1) + MID$(buf, i + 2)            '// cut out the command line switch
  END IF
  i = INSTR(UCASE$(buf), "/H")                          '// look for /H (human output) command line switch
  IF i > 0 THEN
    show = %show_human
    buf = LEFT$(buf, i-1) + MID$(buf, i + 2)            '// cut out the command line switch
  END IF
  path = TRIM$(buf)                                      '// remove any leading or trailing white space
  IF exist(path) THEN                                   '// does the path exist?
    i = GetFolderSize(path)                             '// do our work
    IF d THEN STDOUT DATE$ + " ";                       '// do we show the date?
    IF t THEN STDOUT TIME$ + " ";                       '// do we show the time?
    SELECT CASE show
      CASE %show_raw
        STDOUT path + " = " + FORMAT$(i)                '// make it look good for reports
      CASE %show_human
        STDOUT path + " = " + FORMAT$(i,"0,") + " bytes" '// make it look nice for humans
      CASE ELSE '// include show_none
        STDOUT FORMAT$(i)                               '// just the number
    END SELECT
  ELSE                                                  '// path does not exist or command line syntax error
    PRINT "Error: path not found or syntax error. Use DIRSIZE /? for help."
  END IF
END FUNCTION
